# pygopherd -- Gopher-based protocol server in Python
# module: Present a mbox file as if it were a folder.
# Copyright (C) 2002, 2005 John Goerzen
# <jgoerzen@complete.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import SocketServer
import re
import os, stat, os.path, mimetypes
from pygopherd import protocols, gopherentry
from pygopherd.handlers.virtual import Virtual
from pygopherd.handlers.base import VFS_Real
from mailbox import UnixMailbox, Maildir
from stat import *


###########################################################################
# Basic mailbox support
###########################################################################

class FolderHandler(Virtual):
    def getentry(self):
        ## Return my own entry.
        if not self.entry:
            self.entry = gopherentry.GopherEntry(self.getselector(),
                                                 self.config)
            self.entry.settype('1')
            self.entry.setname(os.path.basename(self.getselector()))
            self.entry.setmimetype('application/gopher-menu')
            self.entry.setgopherpsupport(0)
        return self.entry

    def prepare(self):
        self.entries = []
        count = 1
        while 1:
            message = self.mbox.next()
            if not message:
                break
            handler = MessageHandler(self.genargsselector(self.getargflag() + \
                                     str(count)), self.searchrequest,
                                     self.protocol, self.config, None)
            self.entries.append(handler.getentry(message))
            count += 1

    def isdir(self):
        return 1

    def getdirlist(self):
        return self.entries

class MessageHandler(Virtual):
    def canhandlerequest(self):
        """We put MBOX-MESSAGE in here so we don't have to re-check
        the first line of the mbox file before returning a true or false
        result."""
        if not self.selectorargs:
            return 0
        msgnum = re.search('^' + self.getargflag() + '(\d+)$',
                           self.selectorargs)
        if not msgnum:
            return 0
        self.msgnum = int(msgnum.group(1))
        self.message = None
        return 1

    def getentry(self, message = None):
        """Set the message if called from, eg, the dir handler.  Saves
        having to rescan the file.  If not set, will figure it out."""
        if not message:
            message = self.getmessage()
            
        if not self.entry:
            self.entry = gopherentry.GopherEntry(self.selector, self.config)
            self.entry.settype('0')
            self.entry.setmimetype('text/plain')
            self.entry.setgopherpsupport(0)

            subject = message.getheader('Subject', '<no subject>')
            # Sanitize, esp. for continuations.
            subject = re.sub('\s+', ' ', subject)
            if subject:
                self.entry.setname(subject)
            else:
                self.entry.setname('<no subject>')
        return self.entry

    def getmessage(self):
        if self.message:
            return self.message
        mbox = self.openmailbox()
        message = None
        for x in range(self.msgnum):
            message = mbox.next()
        self.message = message
        return self.message

    def prepare(self):
        self.canhandlerequest()         # Init the vars

    def write(self, wfile):
        # Print out the headers first.
        for header in self.getmessage().headers:
            wfile.write(header)

        # Now the message body.
        self.rfile = self.getmessage().fp
        while 1:
            string = self.rfile.read(4096)
            if not len(string):
                break
            wfile.write(string)
        self.rfile.close()
        self.rfile = None

###########################################################################
# Unix MBOX support
###########################################################################

class MBoxFolderHandler(FolderHandler):
    def canhandlerequest(self):
        """Figure out if this is a handleable request."""

        if self.selectorargs:
            return 0
        
        if not (self.statresult and S_ISREG(self.statresult[ST_MODE])):
            return 0
        try:
            fd = self.vfs.open(self.getselector(), "rt")
            startline = fd.readline()
            fd.close()
            
            return re.match(UnixMailbox._fromlinepattern, startline)
        except IOError:
            return 0

    def prepare(self):
        self.rfile = self.vfs.open(self.getselector(), "rt")
        self.mbox = UnixMailbox(self.rfile)
        FolderHandler.prepare(self)

    def getargflag(self):
        return "/MBOX-MESSAGE/"

class MBoxMessageHandler(MessageHandler):
    def getargflag(self):
        return "/MBOX-MESSAGE/"

    def openmailbox(self):
        fd = self.vfs.open(self.getselector(), "rt")
        return UnixMailbox(fd)

###########################################################################
# Maildir support
###########################################################################

class MaildirFolderHandler(FolderHandler):
    def canhandlerequest(self):
        if not isinstance(self.vfs, VFS_Real):
            return 0
        if self.selectorargs:
            return 0
        if not (self.statresult and S_ISDIR(self.statresult[ST_MODE])):
            return 0
        return self.vfs.isdir(self.getselector() + "/new") and \
               self.vfs.isdir(self.getselector() + "/cur")

    def prepare(self):
        self.mbox = Maildir(self.getfspath())
        FolderHandler.prepare(self)

    def getargflag(self):
        return "/MAILDIR-MESSAGE/"

class MaildirMessageHandler(MessageHandler):
    def getargflag(self):
        return "/MAILDIR-MESSAGE/"

    def openmailbox(self):
        return Maildir(self.getfspath())




# pygopherd -- Gopher-based protocol server in Python
# module: Implementation of features first found in UMN gopherd
# Copyright (C) 2002 John Goerzen
# <jgoerzen@complete.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import SocketServer
import re
import os, stat, os.path, mimetypes
from pygopherd import protocols, gopherentry
from pygopherd.gopherentry import GopherEntry
from pygopherd.handlers.dir import DirHandler
from pygopherd.handlers.file import FileHandler
from stat import *
import pygopherd.fileext

extstrip = None

###########################################################################
# UMN Directory handler
# Handles .Links, .names, and .cap/* files
###########################################################################

class LinkEntry(GopherEntry):
    def __init__(self, selector, config):
        GopherEntry.__init__(self, selector, config)
        self.needsmerge = 0
        self.needsabspath = 0
    def getneedsmerge(self):
        return self.needsmerge
    def getneedsabspath(self):
        return self.needsabspath
    def setneedsmerge(self, arg):
        self.needsmerge = arg
    def setneedsabspath(self, arg):
        self.needsabspath = arg

class UMNDirHandler(DirHandler):
    """This module strives to be bug-compatible with UMN gopherd."""

    def prepare(self):
        """Override parent to do a few more things and override sort order."""
        # Initialize.
        self.linkentries = []

        # Let the parent do the directory walking for us.  Will call
        # prep_initfiles_canaddfile and prep_entriesappend.
        if DirHandler.prepare(self):
            # Returns 1 if it didn't load from the cache.
            # Merge and sort.
            self.MergeLinkFiles()
            self.fileentries.sort(self.entrycmp)
        
    def prep_initfiles_canaddfile(self, ignorepatt, pattern, file):
        """Override the parent to process dotfiles and keep them out
        of the list."""
        if DirHandler.prep_initfiles_canaddfile(self, ignorepatt, pattern,
                                                 file):
            # If the parent says it's OK, then let's see if it's
            # a link file.  If yes, process it and return false.
            if file[0] == '.':
                if not self.vfs.isdir(self.selectorbase + '/' + file):
                    self.linkentries.extend(self.processLinkFile(self.selectorbase + '/' + file))
                    return 0
                else:
                    return 0            # A "dot dir" -- ignore.
            return 1                    # Not a dot file -- return true
        else:
            return 0                    # Parent returned 0, do the same.

    def prep_entriesappend(self, file, handler, fileentry):
        """Overridden to process .cap files and modify extensions.
        This is called by the
        parent's prepare to append an entry to the list.  Here, we check
        to see if there's a .cap file right before adding it."""

        global extstrip
        if extstrip == None:
            extstrip = self.config.get("handlers.UMN.UMNDirHandler",
                                       "extstrip")
        if extstrip != 'none' and \
               isinstance(handler, FileHandler):
            if extstrip == 'full' or \
               (extstrip == 'nonencoded' and not fileentry.getencoding()):
                # If it's a file, has a MIME type, and we know about it..
                fileentry.setname(
                    pygopherd.fileext.extstrip(file,
                                               fileentry.getencodedmimetype() or
                                               fileentry.getmimetype()))
        
        capfilename = self.selectorbase + '/.cap/' + file
        
        try:
            capinfo = self.processLinkFile(capfilename,
                                           fileentry.getselector())
            if len(capinfo) >= 1:       # We handle one and only one entry.
                if capinfo[0].gettype() == 'X' or capinfo[0].gettype() == '-':
                    return              # Type X -- don't append.
                else:
                    self.mergeentries(fileentry, capinfo[0])
        except IOError:                 # Ignore no capfile situation
            pass
        DirHandler.prep_entriesappend(self, file, handler, fileentry)

    def MergeLinkFiles(self):
        """Called to merge the files from .Links and .names into the
        objects obtained by walking the directory.  According to UMN code,
        we ONLY merge if the Path starts with ./ or ~/ in the file.  This
        is set in the getneedsmerge() attribute.  If that attribute is
        not set, don't even bother with it -- just add."""

        # For faster matching, make a dictionary out of the list.
        
        fileentriesdict = {}
        for entry in self.fileentries:
            fileentriesdict[entry.selector] = entry

        for linkentry in self.linkentries:
            if not linkentry.getneedsmerge():
                self.fileentries.append(linkentry)
                continue
            if fileentriesdict.has_key(linkentry.selector):
                if linkentry.gettype() == 'X':
                    # It's special code to hide something.
                    self.fileentries.remove(fileentriesdict[linkentry.selector])
                else:
                    self.mergeentries(fileentriesdict[linkentry.selector],
                                      linkentry)
            else:
                self.fileentries.append(linkentry)

    def mergeentries(self, old, new):
        """Takes the set fields from new and modifies old to have their
        value."""
        for field in ['selector', 'type', 'name', 'host', 'port']:
            if getattr(new, field):
                setattr(old, field, getattr(new, field))

        for field in new.geteadict().keys():
            old.setea(field, new.getea(field))

    def processLinkFile(self, filename, capfilepath = None):
        """Processes a link file.  If capfilepath is set, it should
        be the equivolent of the Path= in a .names file."""
        linkentries = []
        fd = self.vfs.open(filename, "rt")
        while 1:
            nextstep, entry = self.getLinkItem(fd, capfilepath)
            if entry:
                linkentries.append(entry)
            if nextstep == 'stop':
                break
        return linkentries
        
    def getLinkItem(self, fd, capfilepath = None):
        """This is an almost exact clone of UMN's GSfromLink function."""
        entry = LinkEntry(self.entry.selector, self.config)
        nextstep = 'continue'

        done = {'path' : 0, 'type' : 0, 'name' : 0, 'host' : 0, 'port' : 0}

        if capfilepath != None:
            entry.setselector(capfilepath)
            done['path'] = 1

        while 1:
            line = fd.readline()
            if not line:
                nextstep = 'stop'
                break
            line = line.strip()

            # Empty.
            if len(line) == 0:
                break

            # Comment.
            if line[0] == '#':
                if done['path']:
                    break
                else:
                    continue

            # Type.
            if line[0:5] == "Type=":
                entry.settype(line[5])
                # FIXME: handle if line[6] is + or ?
                done['type'] = 1
            elif line[0:5] == "Name=":
                entry.setname(line[5:])
                done['name'] = 1
            elif line[0:5] == "Path=":
                pathname = line[5:]
                if len(pathname) and pathname[-1] == '/':
                    pathname = pathname[0:-1]
                if len(line) >= 7 and (line[5:7] == './' or line[5:7] == '~/'):
                    # Handle ./: make full path.
                    entry.setselector(self.selectorbase + "/" + pathname[2:])
                    entry.setneedsmerge(1)
                elif len(pathname) and pathname[0] != '/':
                    entry.setselector(pathname)
                    entry.setneedsabspath(1)
                else:
                    entry.setselector(pathname)
                done['path'] = 1
            elif line[0:5] == 'Host=':
                if line[5:] != '+':
                    entry.sethost(line[5:])
                done['host'] = 1
            elif line[0:5] == 'Port=':
                if line[5:] != '+':
                    entry.setport(int(line[5:]))
                done['port'] = 1
            elif line[0:5] == 'Numb=':
                entry.setnum(int(line[5:]))
            elif line[0:9] == 'Abstract=':
                abstractstr = ""
                abstractline = line[9:]
                while len(abstractline) and abstractline[-1] == "\\":
                    abstractstr += abstractline[0:-1] + "\n"
                    abstractline = fd.readline().strip()
                abstractstr += abstractline

                if abstractstr:
                    entry.setea('ABSTRACT', abstractstr)
            elif line[0:6] == 'Admin=' or \
                 line[0:4] == 'URL=' or \
                 line[0:4] == 'TTL=':
                pass
            else:
                break
            ### FIXME: Handle Admin, URL, TTL

        if done['path']:
            if entry.getneedsabspath() and \
                   entry.gethost() == None and entry.getport() == None:
                entry.setselector(os.path.normpath(self.selectorbase + "/" + entry.getselector()))
            return (nextstep, entry)
        return (nextstep, None)

    def sgn(self, a):
        """Returns -1 if less than 0, 1 if greater than 0, and 0 if
        equal to zero."""
        if a == 0:
            return 0
        if a < 0:
            return -1
        return 1

    def entrycmp(self, entry1, entry2):
        """This function implements an exact replica of UMN behavior
        GSqsortcmp() behavior."""
        if entry1.name == None:
            return 1
        if entry2.name == None:
            return -1
        e1num = entry1.getnum(0)
        e2num = entry2.getnum(0)

        # Equal numbers or no numbers: sort by title.
        if e1num == e2num:
            return cmp(entry1.name, entry2.name)

        # Same signs: use plain numeric comparison.
        if (self.sgn(e1num) == self.sgn(e2num)):
            return cmp(e1num, e2num)

        # Different signs: other comparison.
        if e1num > e2num:
            return -1
        else:
            return 1




# Python-based gopher server
# Module: signal handlers
# COPYRIGHT #
# Copyright (C) 2002 John Goerzen
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# END OF COPYRIGHT #

import signal, sys, os
from pygopherd import logger

pgrp = None
pid = None

def huphandler(signum, frame):
    logger.log("SIGHUP (%d) received; terminating process" % signum)
    os._exit(5)                         # So we don't raise SystemExit

def termhandler(signum, frame):
    if os.getpid() == pid:              # Master killed; kill children.
        logger.log("SIGTERM (%d) received in master; doing orderly shutdown" \
                   % signum)
        logger.log("Terminating all of process group %d with SIGHUP" % \
                   (pgrp))
        # Ignore this signal so that our own process won't get it again.
        signal.signal(signal.SIGHUP, signal.SIG_IGN)
        os.kill(0, signal.SIGHUP)
        logger.log("Master application process now exiting.  Goodbye.")
        sys.exit(6)
    else:                               # Shouldn't need this -- just in case.
        logger.log("SIGTERM (%d) received in child; terminating this process" \
                   % signum)
        os._exit(7)                     # So we don't raise SystemExit

def setsighuphandler():
    if 'SIGHUP' in signal.__dict__:
        signal.signal(signal.SIGHUP, huphandler)

def setsigtermhandler(newpgrp):
    global pgrp, pid
    pgrp = newpgrp
    pid = os.getpid()
    signal.signal(signal.SIGTERM, termhandler)





# pygopherd -- Gopher-based protocol server in Python
# module: server entry point
# Copyright (C) 2002 John Goerzen
# <jgoerzen@complete.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import SocketServer
import re
import os, stat, os.path, mimetypes, time
from pygopherd import handlers, protocols, GopherExceptions
from pygopherd.protocols.rfc1436 import GopherProtocol

class GopherPlusProtocol(GopherProtocol):
    """Implementation of Gopher+ protocol.  Will handle Gopher+
    queries ONLY."""

    def canhandlerequest(self):
        """We can handle the request IF:
           * It has more than one parameter in the request list
           * The second parameter is ! or starts with + or $"""
        if len(self.requestlist) < 2:
            return 0
        if len(self.requestlist) == 2:
            self.gopherpstring = self.requestlist[1]
        elif len(self.requestlist) == 3:
            self.gopherpstring = self.requestlist[2]
            self.searchrequest = self.requestlist[1]
        else:
            return 0                    # Too many params.

        return self.gopherpstring[0] == '+' or \
               self.gopherpstring == '!' or \
               self.gopherpstring[0] == '$'
    
    def handle(self):
        """Handle Gopher+ request."""
        self.handlemethod = None
        if self.gopherpstring[0] == '+':
            self.handlemethod = 'documentonly'
        elif self.gopherpstring == '!':
            self.handlemethod = 'infoonly'
        elif self.gopherpstring[0] == '$':
            self.handlemethod = 'gopherplusdir'

        try:
            handler = self.gethandler()
            self.log(handler)
            self.entry = handler.getentry()
            
            if self.handlemethod == 'infoonly':
                self.wfile.write("+-2\r\n")
                self.wfile.write(self.renderobjinfo(self.entry))
            else:
                handler.prepare()
                self.wfile.write("+" + str(self.entry.getsize(-2)) + "\r\n")
                if handler.isdir():
                    self.writedir(self.entry, handler.getdirlist())
                else:
                    handler.write(self.wfile)
        except GopherExceptions.FileNotFound, e:
            self.filenotfound(str(e))
        except IOError, e:
            GopherExceptions.log(e, self, None)
            self.filenotfound(e[1])

    def getsupportedblocknames(self, entry):
        # Return the always-supported values PLUS any extra ones for
        # this particular entry.
        return ['+INFO', '+ADMIN', '+VIEWS'] + \
               ['+' + x for x in entry.geteadict().keys()]

    def getallblocks(self, entry):
        retstr = ''
        for block in self.getsupportedblocknames(entry):
            retstr += self.getblock(block, entry)
        return retstr

    def getblock(self, block, entry):
        # If the entry has the block in its eadict, return that.
        # Otherwise, do our own thing.
        # Incoming block: +VIEWS
        blockname = block[1:].lower()

        if entry.geteadict().has_key(blockname.upper()):
            return "+" + blockname.upper() + ":\r\n" + \
                   ''.join(
                           [" " + x + "\r\n" for x in \
                           entry.getea(blockname.upper()).splitlines()]
                   )

        # Not in there -- look up a custom function.
        
        # Name: views
        funcname = "get" + blockname + "block"
        # Funcname: getviewsblock
        func = getattr(self, funcname)
        return func(entry)

    def getinfoblock(self, entry):
        return "+INFO: " + \
               GopherProtocol.renderobjinfo(self, entry)

    def getadminblock(self, entry):
        retstr = "+ADMIN:\r\n"
        retstr += " Admin: "
        retstr += self.config.get("protocols.gopherp.GopherPlusProtocol",
                                  "admin")
        retstr += "\r\n"
        if entry.getmtime():
            retstr += " Mod-Date: "
            retstr += time.ctime(entry.getmtime())
            m = time.localtime(entry.getmtime())
            retstr += " <%04d%02d%02d%02d%02d%02d>\r\n" % \
                      (m[0], m[1], m[2], m[3], m[4], m[5])
        return retstr

    def getviewsblock(self, entry):
        retstr = ''
        if entry.getmimetype():
            retstr += "+VIEWS:\r\n " + entry.getmimetype()
            if (entry.getlanguage()):
                retstr += " " + entry.getlanguage()
            retstr += ":"
            if (entry.getsize() != None):
                retstr += " <%dk>" % (entry.getsize() / 1024)
            retstr += "\r\n"
        return retstr

    def renderobjinfo(self, entry):
        if entry.getmimetype('FAKE') == 'application/gopher-menu' and \
               entry.getgopherpsupport():
            entry.mimetype = 'application/gopher+-menu'
        if self.handlemethod == 'documentonly':
            # It's a Gopher+ request for a gopher0 menu entry.
            retstr = GopherProtocol.renderobjinfo(self, entry)
            return retstr
        else:
            return self.getallblocks(entry)

    def filenotfound(self, msg):
        self.wfile.write("--2\r\n")
        self.wfile.write("1 ")
        self.wfile.write(self.config.get("protocols.gopherp.GopherPlusProtocol", "admin"))
        self.wfile.write("\r\n" + msg + "\r\n")

    def groksabstract(self):
        return 1

class URLGopherPlus(GopherPlusProtocol):
    def getsupportedblocknames(self):
        return GopherPlusProtocol.getsupportedblocknames(self) + \
               ['+URL']

    def geturlblock(self, entry):
        return "+URL: %s\r\n" % entry.geturl(self.server.server_name,
                                            self.server.server_port)



#!/usr/bin/env python2.3

# $Id: setup.py,v 1.6 2002/04/11 21:12:12 jgoerzen Exp $

# Python-based gopher server
# Module: installer
# COPYRIGHT #
# Copyright (C) 2002 John Goerzen
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# END OF COPYRIGHT #


from distutils.core import setup
import pygopherd.version

setup(name = "pygopherd",
      version = pygopherd.version.versionstr,
      description = pygopherd.version.description,
      author = pygopherd.version.author,
      author_email = pygopherd.version.author_email,
      url = pygopherd.version.homepage,
      packages = ['pygopherd', 'pygopherd.handlers', 'pygopherd.protocols'],
      scripts = ['bin/pygopherd'],
      data_files = [ ('/etc/pygopherd', ['conf/pygopherd.conf',
                                         'conf/mime.types'] ) ],
      license = pygopherd.version.license
)




#!/usr/bin/python

# Python-based gopher server
# Module: test of Gopher Exceptions
# COPYRIGHT #
# Copyright (C) 2002 John Goerzen
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# END OF COPYRIGHT #

import unittest
from StringIO import StringIO
from pygopherd import logger, initialization, GopherExceptions, testutil
from pygopherd.GopherExceptions import FileNotFound
from pygopherd.protocols import rfc1436

class GopherExceptionsTestCase(unittest.TestCase):
    def setUp(self):
        self.stringfile = StringIO()
        self.config = testutil.getconfig()
        self.stringfile = testutil.getstringlogger()
        GopherExceptions.tracebacks = 0

    def testlog_basic(self):
        try:
            raise IOError, "foo"
        except IOError, e:
            GopherExceptions.log(e)
        self.assertEqual(self.stringfile.getvalue(),
                         "unknown-address [None/None] EXCEPTION IOError: foo\n")

    def testlog_proto_ip(self):
        rfile = StringIO("/NONEXISTANT\n")
        wfile = StringIO()
        handler = testutil.gettestinghandler(rfile, wfile, self.config)
        handler.handle()
        # handler.handle()
        self.assertEquals(self.stringfile.getvalue(),
             "10.77.77.77 [GopherProtocol/None] EXCEPTION FileNotFound: '/NONEXISTANT' does not exist (no handler found)\n")

    def testFileNotFound(self):
        try:
            raise FileNotFound, "TEST STRING"
        except FileNotFound, e:
            self.assertEquals(str(e),
                              "'TEST STRING' does not exist")




#!/usr/bin/python

# Python-based gopher server
# Module: test of fileext
# COPYRIGHT #
# Copyright (C) 2002 John Goerzen
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# END OF COPYRIGHT #

import unittest
from pygopherd import fileext, testutil, initialization

class FileExtTestCase(unittest.TestCase):
    def setUp(self):
        config = testutil.getconfig()
        initialization.initmimetypes(config)
        
    def testinit(self):
        # Was already inited in the initmimetypes, so just do a sanity
        # check.
        self.assert_('.txt' in fileext.typemap['text/plain'])
        self.assert_('.txt.gz' in fileext.typemap['text/plain'])
        self.assert_(not ('.html' in fileext.typemap['text/plain']))
        



productname = 'PyGopherd'
versionstr = "2.0.18"

versionlist = versionstr.split(".")
major = versionlist[0]
minor = versionlist[1]
patch = versionlist[2]
copyright = "Copyright (C) 2002-2008 John Goerzen.  All rights reserved."
author = "John Goerzen"
author_email = "jgoerzen@complete.org"
description = "Multiprotocol Internet Gopher Information Server"
homepage = "http://www.quux.org/devel/gopher/pygopherd"
homegopher = "gopher://quux.org/1/devel/gopher/pygopherd"
license = """Copyright (C) 2002-2008 John Goerzen <jgoerzen@complete.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"""



# pygopherd -- Gopher-based protocol server in Python
# module: protocol directory index
# Copyright (C) 2002 John Goerzen
# <jgoerzen@complete.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ['base', 'enhanced', 'gopherp', 'rfc1436', 'http', 'wap']



# pygopherd -- Gopher-based protocol server in Python
# module: Generic gopher entry object
# Copyright (C) 2002 John Goerzen
# <jgoerzen@complete.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import SocketServer
import re
import os, stat, os.path, mimetypes, urllib

mapping = None
eaexts = None

class GopherEntry:
    """The entry object for Gopher.  It holds information about each
    Gopher object."""

    def __init__(self, selector, config):
        """Initialize object based on a selector and config."""
        self.selector = selector        # Gopher path to file
        self.config = config            # Our config object
        self.fspath = None              # Path to the obj in filesystem
        self.type = None                # Gopher0 type char
        self.name = None                # Menu name
        self.host = None                # Hostname
        self.port = None                # Port number (an int)
        self.mimetype = None            # MIME type
        self.encodedmimetype = None     # Real MIME type if encoded.
        self.size = None                # Size
        self.encoding = None            # Encoding type
        self.populated = 0              # Whether or not it's been populated
        self.language = None            # Language
        self.ctime = None               # Creation date
        self.mtime = None               # Modification date
        self.num = 0                    # Number in menu
        self.gopherpsupport = 0         # Supports gopher+
        self.ea = {}                    # Extended attributes -- Gopher+
                                        # Abstract, etc.

    def populatefromvfs(self, vfs, selector):
        self.populatefromfs(selector, statval = vfs.stat(selector),
                            vfs = vfs)

    def populatefromfs(self, fspath, statval = None, vfs = None):
        """Fills in self with data gleaned from the filesystem.

        The argument fspath specifies where in the filesystem it will search.
        statval, if present, will be used instead of calling stat() itself
        so as to cut down on the number of system calls.

        The overall idea of this function is to set only those values that
        are not already set and to do so only if a definitive answer for
        them can be obtained from the operating system.  The rest of this
        information describes the details of how it does this.

        If populatefromfs has already been called or self.populated is true,
        this function will note the fspath and the return without modifying
        anything else.

        If either gethost() or getport() return anything other than None,
        the same thing will happen.

        If there is no statval and os.stat returns an error, again,
        the same thing will happen.

        Assuming these tests pass, then:

        self.gopherpsupport will be set to true for any file or directory.

        The remaining values will be set only if they are not set already:

        For both files and directories, the creation and modification times
        will be noted, the name will be set to the filename (as returned
        by os.path.basename on the selector).

        For directories only, the type will be set to 1 and the mimetype to
        application/gopher-menu.  Gopher+ protocols may wish to
        indicate application/gopher+-menu is available as well, but that
        is outside the scope of this function.

        For files only, the size will be noted.  An attempt will be made
        to ascertain a mimetype and an encoding.  If only a mimetype is
        found, it will be noted in self.mimetype.  If both a mimetype
        and an encoding is found, self.mimetype will be
        application/octet-stream, self.encoding will note the encoding,
        and self.encodedmimetype will note the type of the encoded data.
        If no mimetype can be found, it will be set to the default
        from the config file.  If no gopher0 type character is already present,
        self.guesstype() will be called to set it."""
        
        self.fspath = fspath
        if vfs == None:
            from pygopherd.handlers.base import VFS_Real
            vfs = VFS_Real(self.config)

        if self.populated:
            return

        # Just let the stat catch the OSError rather than testing
        # for existance here.  Help cut down on the number of syscalls.

        if not (self.gethost() == None and self.getport() == None):
            return

        if not statval:
            try:
                statval = vfs.stat(self.fspath)
            except OSError:
                return
        
        self.populated = 1
        self.gopherpsupport = 1         # Indicate gopher+ support for locals.

        # All this "or" stuff means that we only modify it if it's not already
        # set.

        self.ctime = self.ctime or statval[9]
        self.mtime = self.mtime or statval[8]
        self.name = self.name or os.path.basename(self.selector)

        if stat.S_ISDIR(statval[0]):
            self.type = self.type or '1'
            self.mimetype = self.mimetype or 'application/gopher-menu'
            self.handleeaext(self.fspath + '/', vfs) # Add the / so we get /.abs
            return

        self.handleeaext(self.fspath, vfs)

        self.size = self.size or statval[6]

        mimetype, encoding = mimetypes.guess_type(self.selector, strict = 0)

        if encoding:
            self.mimetype = self.mimetype or 'application/octet-stream'
            self.encoding = self.encoding or encoding
            self.encodedmimetype = self.encodedmimetype or mimetype
        else:
            self.mimetype = self.mimetype or mimetype

        # Did we get no mime type at all?  Fall back to a default.

        if not self.mimetype:
            self.mimetype = self.config.get("GopherEntry", "defaultmimetype")

        self.type = self.type or self.guesstype()

    def guesstype(self):
        global mapping
        if not mapping:
            mapping = eval(self.config.get("GopherEntry", "mapping"))
        for maprule in mapping:
            if re.match(maprule[0], self.mimetype):
                return maprule[1]
        return '0'

    def handleeaext(self, selector, vfs):
        """Handle getting extended attributes from the filesystem."""
        global eaexts
        if eaexts == None:
            eaexts = eval(self.config.get("GopherEntry", "eaexts"))
        if vfs == None:
            from pygopherd.handlers.base import VFS_Real
            vfs = VFS_Real(self.config)

        for extension, blockname in eaexts.items():
            if self.ea.has_key(blockname):
                continue
            try:
                rfile = vfs.open(selector + extension, "rt")
                self.setea(blockname, "\n".join(
                           [x.rstrip() for x in rfile.readlines(20480)]))
            except IOError:
                pass
                         

    def getselector(self, default = None):
        if self.selector == None:
            return default
        return self.selector
    def setselector(self, arg):
        self.selector = arg
    def getconfig(self, default = None):
        return self.config or default
    def setconfig(self, arg):
        self.config = arg
    def getfspath(self, default = None):
        if self.fspath == None:
            return default
        return self.fspath
    def setfspath(self, arg):
        self.fspath = arg
    def gettype(self, default = None):
        if self.type == None:
            return default
        return self.type
    def settype(self, arg):
        self.type = arg
    def getname(self, default = None):
        if self.name == None:
            return default
        return self.name
    def setname(self, arg):
        self.name = arg
    def gethost(self, default = None):
        if self.host == None:
            return default
        return self.host
    def sethost(self, arg):
        self.host = arg
    def getport(self, default = None):
        if self.port == None:
            return default
        return self.port
    def setport(self, arg):
        self.port = arg
    def getmimetype(self, default = None):
        if self.mimetype == None:
            return default
        return self.mimetype
    def getencodedmimetype(self, default = None):
        if self.encodedmimetype == None:
            return default
        return self.encodedmimetype
    def setencodedmimetype(self, arg):
        self.encodedmimetype = arg
    def setmimetype(self, arg):
        self.mimetype = arg
    def getsize(self, default = None):
        if self.size == None:
            return default
        return self.size
    def setsize(self, arg):
        self.size = arg
    def getencoding(self, default = None):
        if self.encoding == None:
            return default
        return self.encoding
    def setencoding(self, arg):
        self.encoding = arg
    def getlanguage(self, default = None):
        if self.language == None:
            return default
        return self.language
    def setlanguage(self, arg):
        self.language = arg
    def getctime(self, default = None):
        if self.ctime == None:
            return default
        return self.ctime
    def setctime(self, arg):
        self.ctime = arg
    def getmtime(self, default = None):
        if self.mtime == None:
            return default
        return self.mtime
    def setmtime(self, arg):
        self.mtime = arg
    def getpopulated(self, default = None):
        if self.populated != None:
            return self.populated
        return default
    def setpopulated(self, arg):
        self.populated = arg
    
    def geturl(self, defaulthost = 'MISSINGHOST', defaultport = 70):
        """If this selector is a URL: one, then we just return the rest of
        it.  Otherwise, generate a gopher:// URL and quote it."""
        if re.search("^(/|)URL:.+://", self.selector):
            if self.selector[0] == '/':
                return self.selector[5:]
            else:
                return self.selector[4:]
            
        retval = 'gopher://%s:%d/' % (self.gethost(defaulthost),
                                      self.getport(defaultport))
        retval += urllib.quote('%s%s' % (self.gettype(), self.getselector()))
        return retval

    def getnum(self, default = None):
        if self.num != None:
            return self.num
        return default
    def setnum(self, arg):
        self.num = arg

    def getgopherpsupport(self, default = None):
        if self.gopherpsupport != None:
            return self.gopherpsupport
        return default
    def setgopherpsupport(self, arg):
        self.gopherpsupport = arg

    def getea(self, name, default = None):
        if self.ea.has_key(name):
            return self.ea[name]
        return default

    def geteadict(self):
        return self.ea

    def setea(self, name, value):
        self.ea[name] = value

def getinfoentry(text, config):
    entry = GopherEntry('fake', config)
    entry.name = text
    entry.host = '(NULL)'
    entry.port = 0
    entry.type = 'i'
    return entry



# pygopherd -- Gopher-based protocol server in Python
# module: Execute children in a pipe.
# Copyright (C) 2002 John Goerzen
# <jgoerzen@complete.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, sys

# Later we will check sys.platform

def pipedata_unix(file, args, environ = os.environ,
                  childstdin = None,
                  childstdout = None,
                  childstderr = None,
                  pathsearch = 0):
    pid = os.fork()
    if pid:
        # Parent.
        return os.waitpid(pid, 0)[1]
    else:
        # Child.
        if childstdin:
            os.dup2(childstdin.fileno(), 0)
        if childstdout:
            os.dup2(childstdout.fileno(), 1)
        if childstderr:
            os.dup2(childstderr.fileno(), 2)
        if pathsearch:
            os.execvpe(file, args, environ)
        else:
            os.execve(file, args, environ)
        sys.exit(255)

pipedata = pipedata_unix
